'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _get2 = require('babel-runtime/helpers/get');

var _get3 = _interopRequireDefault(_get2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _weakMap = require('babel-runtime/core-js/weak-map');

var _weakMap2 = _interopRequireDefault(_weakMap);

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @private
 * @type {RegExp}
 */
var DEFAULT_INTERPOLATE = /\[([\w\s]+?)]/g;

/**
 * @private
 * @type {WeakMap}
 */
var INTERPOLATE = new _weakMap2.default();

/**
 * @class
 * @extends {Map}
 */

var ConfigPatternCache = function (_Map) {
    (0, _inherits3.default)(ConfigPatternCache, _Map);

    /**
     * @constructor
     * @param {RegExp} [interpolate=/\[([\w\s]+?)]/g]
     */
    function ConfigPatternCache() {
        var interpolate = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_INTERPOLATE;
        (0, _classCallCheck3.default)(this, ConfigPatternCache);

        var _this = (0, _possibleConstructorReturn3.default)(this, (ConfigPatternCache.__proto__ || (0, _getPrototypeOf2.default)(ConfigPatternCache)).call(this));

        _this.interpolate = interpolate;
        return _this;
    }

    /**
     * @type {RegExp}
     */


    (0, _createClass3.default)(ConfigPatternCache, [{
        key: 'getOrSet',


        /**
         * @param {*} key
         * @returns {RegExp}
         */
        value: function getOrSet(key) {
            if (!this.has(key)) {
                this.set(key, key);
            }

            return this.get(key);
        }

        /**
         * @override
         * @param {*} key
         * @param {String} value
         * @returns {RegExp}
         */

    }, {
        key: 'set',
        value: function set(key, value) {
            return (0, _get3.default)(ConfigPatternCache.prototype.__proto__ || (0, _getPrototypeOf2.default)(ConfigPatternCache.prototype), 'set', this).call(this, key, this.compile(value));
        }

        /**
         * @param {String} value
         * @param {Object} options
         * @returns {String}
         */

    }, {
        key: 'eval',
        value: function _eval(value) {
            var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

            var compiledTemplate = this.getOrSet(value);

            return compiledTemplate(options);
        }

        /**
         * @param {String} value
         * @returns {Function}
         */

    }, {
        key: 'compile',
        value: function compile(value) {
            return (0, _lodash.template)(value, {
                interpolate: this.interpolate
            });
        }
    }, {
        key: 'interpolate',
        get: function get() {
            return INTERPOLATE.get(this);
        }

        /**
         * @example
         * import {
         *   patternCache
         * } from 'webpack-config';
         *
         * patternCache.interpolate = /{([\w\s]+?)}/g;
         * @param {RegExp} value
         */
        ,
        set: function set(value) {
            INTERPOLATE.set(this, value);
        }
    }]);
    return ConfigPatternCache;
}(_map2.default);

exports.default = ConfigPatternCache;
module.exports = exports['default'];
//# sourceMappingURL=ConfigPatternCache.js.map